==> luna3-2.13.60.tar.gz: luna3-2.13.60/luna3/configurator/configurator.py <==
"""
Luna-client client module
"""
from numbers import Real
from typing import Awaitable, Dict, List, Optional, Union

import aiohttp

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload


class ConfiguratorApi(BaseHTTPAPI):
    """
    Class for request to luna-configurator.

    """

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5070",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )

    def pullConfig(
        self, serviceName: str, tags: Optional[Dict[str, str]] = None, accept: str = "json", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Returns an actual config which corresponding  a service and filters.

        Args:
            serviceName: service name
            tags: dict with pair(s) of setting name and tag for it
            accept: value for accept header (json or x-vl-config-file)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with config for service.

        """
        headers = {"Accept": f"application/{accept}"}
        return self.makeRequest(f"{self.baseUri}/puller/{serviceName}", "POST", body=tags, headers=headers, **kwargs)

    def getSettings(
        self,
        serviceName: Optional[str] = None,
        settingName: Optional[str] = None,
        tags: Optional[str] = None,
        description: Optional[str] = None,
        getSchema: Optional[int] = None,
        groupList: Optional[List[str]] = None,
        isDefault: Optional[bool] = None,
        page: int = 1,
        pageSize: int = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get settings with filters and pagination.

        Args:
            page: page
            pageSize: page size
            serviceName: service name
            settingName: setting name
            description: part or whole description to search by it
            groupList: list of groups which settings apply to
            tags: string with tags
            isDefault: get default settings or not
            getSchema: get settings schemas or not

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        queryParams = {
            "page": page,
            "page_size": pageSize,
            "service_name": serviceName,
            "setting_name": settingName,
            "tags": tags,
            "description": description,
            "group_names": groupList,
            "get_schema": getSchema,
            "is_default": isDefault,
        }

        return self.makeRequest(f"{self.baseUri}/settings", "GET", queryParams=queryParams, **kwargs)

    def createSetting(
        self,
        settingName: str,
        value: Union[dict, list, str, int, float],
        tags: List[str],
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new setting.

        Args:
            settingName: setting name
            value: setting value
            tags: list of tags
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        payload = RequestPayload.buildJson(
            body={"tags": tags, "name": settingName, "value": value, "description": description}
        )
        return self.makeRequest(f"{self.baseUri}/settings", "POST", body=payload, **kwargs)

    def putLimitation(
        self,
        limitationName: str,
        defaultValue: Union[dict, list, str, int, float],
        validationSchema: dict,
        services: List[str],
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Put limitation.

        Args:
            limitationName: limitation name
            defaultValue: default setting value
            validationSchema: new setting validation schema
            services: services which use the limitation
            description: description
            groupList: group list to link

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        data = {
            "validation_schema": validationSchema,
            "default_value": defaultValue,
            "description": description,
            "services": services,
        }
        if groupList:
            data["group_names"] = groupList
        payload = RequestPayload.buildJson(body=data)
        return self.makeRequest(f"{self.baseUri}/settings/limitations/{limitationName}", "PUT", body=payload, **kwargs)

    def patchLimitation(
        self,
        limitationName: str,
        defaultValue: Optional[Union[dict, list, str, int, float]] = None,
        validationSchema: Optional[dict] = None,
        services: Optional[List[str]] = None,
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch limitation.

        Args:
            limitationName: limitation name
            defaultValue: default set
            validationSchema: new setting validation schema
            services: services which use the limitation
            description: description
            groupList: group list to link

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        data = {}
        if validationSchema:
            data["validation_schema"] = validationSchema
        if defaultValue:
            data["default_value"] = defaultValue
        if description:
            data["description"] = description
        if services:
            data["services"] = services
        if groupList:
            data["group_names"] = groupList
        payload = RequestPayload.buildJson(body=data)
        return self.makeRequest(
            f"{self.baseUri}/settings/limitations/{limitationName}", "PATCH", body=payload, **kwargs
        )

    def getLimitations(
        self,
        limitationNames: Optional[List[str]] = None,
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get limitations.

        Args:
            limitationNames: limitations names
            description: part or whole description to search by it
            groupList: list of groups

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        queryParams = {"limitation_names": limitationNames, "description": description, "group_names": groupList}
        return self.makeRequest(f"{self.baseUri}/settings/limitations", "GET", queryParams=queryParams, **kwargs)

    def getLimitation(self, limitationName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get limitation by its name.

        Args:
            limitationName: limitation name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        return self.makeRequest(f"{self.baseUri}/settings/limitations/{limitationName}", "GET", **kwargs)

    def deleteLimitation(self, limitationName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete limitation by its name.

        Args:
            limitationName: limitation name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeRequest(f"{self.baseUri}/settings/limitations/{limitationName}", "DELETE", **kwargs)

    def putSetting(
        self,
        settingId: int,
        settingName: Optional[str] = None,
        value: Optional[Union[dict, list, str, int, float]] = None,
        description: Optional[str] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Rewrite a setting.

        Args:
            settingId: setting id
            settingName: setting name
            value: setting value
            tags: string with tags
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        payload = RequestPayload.buildJson(
            body={"tags": tags, "name": settingName, "value": value, "description": description}
        )
        return self.makeRequest(f"{self.baseUri}/settings/{settingId}", "PUT", body=payload, **kwargs)

    def getSetting(self, settingId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get setting by id.

        Args:
            settingId: setting id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self.makeRequest(f"{self.baseUri}/settings/{settingId}", "GET", **kwargs)

    def removeSetting(self, settingId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove setting.

        Args:
            settingId: setting id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeRequest(f"{self.baseUri}/settings/{settingId}", "DELETE", **kwargs)

    def addGroup(
        self, groupName: str, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Add new group.

        Args:
            groupName: new group name
            description: new group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        data = {"group_name": groupName}
        if description is not None:
            data["description"] = description
        payload = RequestPayload.buildJson(body=data)
        return self.makeRequest(f"{self.baseUri}/groups", "POST", body=payload, **kwargs)

    def updateGroup(self, groupName: str, description: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update group.

        Args:
            groupName: new group name
            description: new group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        payload = RequestPayload.buildJson(body={"description": description})
        return self.makeRequest(f"{self.baseUri}/groups/{groupName}", "PATCH", body=payload, **kwargs)

    def getGroup(self, groupName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get group by name.

        Args:
            groupName: group name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self.makeRequest(f"{self.baseUri}/groups/{groupName}", "GET", **kwargs)

    def getGroups(
        self, limitationNames: Optional[List[str]] = None, groupNames: Optional[List[str]] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get groups.

        Args:
            limitationNames: list of limitation/setting names
            groupNames: list of group names

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        queryParams = {"limitation_names": limitationNames, "group_names": groupNames}
        return self.makeRequest(f"{self.baseUri}/groups", "GET", queryParams=queryParams, **kwargs)

    def removeGroup(self, groupName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove group.

        Args:
            groupName: group name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self.makeRequest(f"{self.baseUri}/groups/{groupName}", "DELETE", **kwargs)

    def linkLimitationsToGroups(
        self, groupNames: List[str], limitationNames: List[str], action: str = "attach", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Link/Unlink limitation(s) to group(s).

        Args:
            groupNames: list of groups
            limitationNames: list of limitations
            action: attach or detach

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(
            body={"action": action, "group_names": groupNames, "limitation_names": limitationNames}
        )
        return self.makeRequest(f"{self.baseUri}/groups_linker", "PATCH", body=payload, **kwargs)

    def getServices(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get services.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self.makeRequest(f"{self.baseUri}/settings/services", "GET", **kwargs)

    def getDumpFile(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get json file with all limitations and settings.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json file with all limitations and settings.

        """

        return self.makeRequest(f"{self.baseUri}/dump", "GET", **kwargs)

==> luna3-2.14.75.tar.gz: luna3-2.14.75/luna3/configurator/configurator.py <==
"""Luna-client client module."""

from numbers import Real
from typing import Awaitable, Callable, Dict, List, Optional, Union

import aiohttp

from ..common.base_api import BaseHTTPAPI
from ..common.luna_response import LunaResponse
from ..common.requests import RequestPayload
from ..common.utils import createBasicAuth


class ConfiguratorApi(BaseHTTPAPI):
    """Class for request to luna-configurator."""

    def __init__(
        self,
        origin: Optional[str] = "http://127.0.0.1:5070",
        api: Optional[int] = 1,
        asyncRequest: Optional[bool] = False,
        lunaRequestId: Optional[str] = None,
        totalTimeout: Real = 60,
        connectTimeout: Optional[Real] = None,
        sockReadTimeout: Optional[Real] = None,
        sockConnectTimeout: Optional[Real] = None,
        *,
        session: aiohttp.ClientSession = None,
        echo: bool = False,
        login: Optional[str] = None,
        password: Optional[str] = None,
        **kwargs,
    ):
        super().__init__(
            origin=origin,
            api=api,
            asyncRequest=asyncRequest,
            lunaRequestId=lunaRequestId,
            totalTimeout=totalTimeout,
            connectTimeout=connectTimeout,
            sockReadTimeout=sockReadTimeout,
            sockConnectTimeout=sockConnectTimeout,
            session=session,
            echo=echo,
            **kwargs,
        )
        self.basicAuth: Optional[Dict] = None
        self._makeRequestContext: Callable

        if login and password and len(login) > 0 and len(password) > 0:
            self.basicAuth = createBasicAuth(login, password)

        self._makeRequestContext = self.makeAuthRequest if self.basicAuth else self.makeRequest

    def makeAuthRequest(self, **requestKwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Prepare auth headers and make request.

        Simply adds headers to the request for basic authorization.

        Args:
            requestKwargs: standard request kwargs.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with config for service.

        """
        requestKwargs.setdefault("headers", {}).update(self.basicAuth)
        return self.makeRequest(**requestKwargs)

    def pullConfig(
        self, serviceName: str, tags: Optional[Dict[str, str]] = None, accept: str = "json", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Returns an actual config which corresponding  a service and filters.

        Args:
            serviceName: service name
            tags: dict with pair(s) of setting name and tag for it
            accept: value for accept header (json or x-vl-config-file)

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with config for service.

        """
        headers = {"Accept": f"application/{accept}"}
        return self._makeRequestContext(
            url=f"{self.baseUri}/puller/{serviceName}", method="POST", body=tags, headers=headers, **kwargs
        )

    def getSettings(
        self,
        serviceName: Optional[str] = None,
        settingName: Optional[str] = None,
        tags: Optional[str] = None,
        description: Optional[str] = None,
        getSchema: Optional[int] = None,
        groupList: Optional[List[str]] = None,
        isDefault: Optional[bool] = None,
        page: int = 1,
        pageSize: int = 10,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get settings with filters and pagination.

        Args:
            page: page
            pageSize: page size
            serviceName: service name
            settingName: setting name
            description: part or whole description to search by it
            groupList: list of groups which settings apply to
            tags: string with tags
            isDefault: get default settings or not
            getSchema: get settings schemas or not

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        queryParams = {
            "page": page,
            "page_size": pageSize,
            "service_name": serviceName,
            "setting_name": settingName,
            "tags": tags,
            "description": description,
            "group_names": groupList,
            "get_schema": getSchema,
            "is_default": isDefault,
        }

        return self._makeRequestContext(url=f"{self.baseUri}/settings", method="GET", queryParams=queryParams, **kwargs)

    def createSetting(
        self,
        settingName: str,
        value: Union[dict, list, str, int, float],
        tags: List[str],
        description: Optional[str] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Create new setting.

        Args:
            settingName: setting name
            value: setting value
            tags: list of tags
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        payload = RequestPayload.buildJson(
            body={"tags": tags, "name": settingName, "value": value, "description": description}
        )
        return self._makeRequestContext(url=f"{self.baseUri}/settings", method="POST", body=payload, **kwargs)

    def putLimitation(
        self,
        limitationName: str,
        defaultValue: Union[dict, list, str, int, float],
        validationSchema: dict,
        services: List[str],
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Put limitation.

        Args:
            limitationName: limitation name
            defaultValue: default setting value
            validationSchema: new setting validation schema
            services: services which use the limitation
            description: description
            groupList: group list to link

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        data = {
            "validation_schema": validationSchema,
            "default_value": defaultValue,
            "description": description,
            "services": services,
        }
        if groupList:
            data["group_names"] = groupList
        payload = RequestPayload.buildJson(body=data)
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/limitations/{limitationName}", method="PUT", body=payload, **kwargs
        )

    def patchLimitation(
        self,
        limitationName: str,
        defaultValue: Optional[Union[dict, list, str, int, float]] = None,
        validationSchema: Optional[dict] = None,
        services: Optional[List[str]] = None,
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Patch limitation.

        Args:
            limitationName: limitation name
            defaultValue: default set
            validationSchema: new setting validation schema
            services: services which use the limitation
            description: description
            groupList: group list to link

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting id.

        """
        data = {}
        if validationSchema:
            data["validation_schema"] = validationSchema
        if defaultValue:
            data["default_value"] = defaultValue
        if description:
            data["description"] = description
        if services:
            data["services"] = services
        if groupList:
            data["group_names"] = groupList
        payload = RequestPayload.buildJson(body=data)
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/limitations/{limitationName}", method="PATCH", body=payload, **kwargs
        )

    def getLimitations(
        self,
        limitationNames: Optional[List[str]] = None,
        description: Optional[str] = None,
        groupList: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get limitations.

        Args:
            limitationNames: limitations names
            description: part or whole description to search by it
            groupList: list of groups

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        queryParams = {"limitation_names": limitationNames, "description": description, "group_names": groupList}
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/limitations", method="GET", queryParams=queryParams, **kwargs
        )

    def getLimitation(self, limitationName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get limitation by its name.

        Args:
            limitationName: limitation name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with settings.

        """
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/limitations/{limitationName}", method="GET", **kwargs
        )

    def deleteLimitation(self, limitationName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Delete limitation by its name.

        Args:
            limitationName: limitation name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/limitations/{limitationName}", method="DELETE", **kwargs
        )

    def putSetting(
        self,
        settingId: int,
        settingName: Optional[str] = None,
        value: Optional[Union[dict, list, str, int, float]] = None,
        description: Optional[str] = None,
        tags: Optional[List[str]] = None,
        **kwargs,
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Rewrite a setting.

        Args:
            settingId: setting id
            settingName: setting name
            value: setting value
            tags: string with tags
            description: description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        payload = RequestPayload.buildJson(
            body={"tags": tags, "name": settingName, "value": value, "description": description}
        )
        return self._makeRequestContext(
            url=f"{self.baseUri}/settings/{settingId}", method="PUT", body=payload, **kwargs
        )

    def getSetting(self, settingId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get setting by id.

        Args:
            settingId: setting id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self._makeRequestContext(url=f"{self.baseUri}/settings/{settingId}", method="GET", **kwargs)

    def removeSetting(self, settingId: int, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove setting.

        Args:
            settingId: setting id

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self._makeRequestContext(url=f"{self.baseUri}/settings/{settingId}", method="DELETE", **kwargs)

    def addGroup(
        self, groupName: str, description: Optional[str] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Add new group.

        Args:
            groupName: new group name
            description: new group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        data = {"group_name": groupName}
        if description is not None:
            data["description"] = description
        payload = RequestPayload.buildJson(body=data)
        return self._makeRequestContext(url=f"{self.baseUri}/groups", method="POST", body=payload, **kwargs)

    def updateGroup(self, groupName: str, description: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Update group.

        Args:
            groupName: new group name
            description: new group description

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return.

        """
        payload = RequestPayload.buildJson(body={"description": description})
        return self._makeRequestContext(
            url=f"{self.baseUri}/groups/{groupName}", method="PATCH", body=payload, **kwargs
        )

    def getGroup(self, groupName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get group by name.

        Args:
            groupName: group name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self._makeRequestContext(url=f"{self.baseUri}/groups/{groupName}", method="GET", **kwargs)

    def getGroups(
        self, limitationNames: Optional[List[str]] = None, groupNames: Optional[List[str]] = None, **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get groups.

        Args:
            limitationNames: list of limitation/setting names
            groupNames: list of group names

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        queryParams = {"limitation_names": limitationNames, "group_names": groupNames}
        return self._makeRequestContext(url=f"{self.baseUri}/groups", method="GET", queryParams=queryParams, **kwargs)

    def removeGroup(self, groupName: str, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Remove group.

        Args:
            groupName: group name

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        return self._makeRequestContext(url=f"{self.baseUri}/groups/{groupName}", method="DELETE", **kwargs)

    def linkLimitationsToGroups(
        self, groupNames: List[str], limitationNames: List[str], action: str = "attach", **kwargs
    ) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Link/Unlink limitation(s) to group(s).

        Args:
            groupNames: list of groups
            limitationNames: list of limitations
            action: attach or detach

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.

        """
        payload = RequestPayload.buildJson(
            body={"action": action, "group_names": groupNames, "limitation_names": limitationNames}
        )
        return self._makeRequestContext(url=f"{self.baseUri}/groups_linker", method="PATCH", body=payload, **kwargs)

    def getServices(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get services.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json with setting.

        """
        return self._makeRequestContext(url=f"{self.baseUri}/settings/services", method="GET", **kwargs)

    def getDumpFile(self, **kwargs) -> Union[Awaitable[LunaResponse], LunaResponse]:
        """
        Get json file with all limitations and settings.

        Returns:
            class:`~.LunaResponse` or *asyncio coroutine* with *LunaResponse*.
            In body of :class: `~.LunaResponse` will return json file with all limitations and settings.

        """

        return self._makeRequestContext(url=f"{self.baseUri}/dump", method="GET", **kwargs)
